package flink.zanxiangnet.ad.monitoring.process;

import com.aliyun.odps.Instance;
import com.aliyun.odps.Odps;
import com.aliyun.odps.account.Account;
import com.aliyun.odps.account.AliyunAccount;
import com.aliyun.odps.data.Record;
import com.aliyun.odps.task.SQLTask;
import flink.zanxiangnet.ad.monitoring.maxcompute.MaxComputeLog;
import flink.zanxiangnet.ad.monitoring.pojo.entity.AdDataOfDayODS;
import flink.zanxiangnet.ad.monitoring.pojo.entity.PlanStatOfDayDWD;
import flink.zanxiangnet.ad.monitoring.pojo.properties.ApplicationProperties;
import flink.zanxiangnet.ad.monitoring.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.common.state.ValueState;
import org.apache.flink.api.common.state.ValueStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.GlobalWindow;
import org.apache.flink.util.Collector;

import java.time.LocalDate;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
public class AdDayDWDRollMonthProcess extends ProcessWindowFunction<AdDataOfDayODS, PlanStatOfDayDWD, Long, GlobalWindow> {
    private Odps odps;
    // 上次查询的时间
    private ValueState<String> lastQueryDayState;
    // 之前聚合的昨天的数据
    private MapState<String, PlanStatOfDayDWD> historyReduceState;

    @Override
    public void open(Configuration conf) throws Exception {
        Map<String, String> params = getRuntimeContext()
                .getExecutionConfig()
                .getGlobalJobParameters()
                .toMap();
        Account account = new AliyunAccount(params.get(ApplicationProperties.MAX_COMPUTE_ACCOUNT_ID),
                params.get(ApplicationProperties.MAX_COMPUTE_ACCOUNT_KEY));
        odps = new Odps(account);
        odps.getRestClient().setRetryLogger(new MaxComputeLog());
        odps.setEndpoint(params.get(ApplicationProperties.MAX_COMPUTE_ACCOUNT_ENDPOINT));
        odps.setDefaultProject(params.get(ApplicationProperties.MAX_COMPUTE_ACCOUNT_PROJECT_NAME));

        lastQueryDayState = getRuntimeContext().getState(new ValueStateDescriptor<>("lastQueryDayState", String.class));
        historyReduceState = getRuntimeContext().getMapState(new MapStateDescriptor<>("historyReduceState", String.class, PlanStatOfDayDWD.class));
    }

    @Override
    public void process(Long elementsCount, ProcessWindowFunction<AdDataOfDayODS, PlanStatOfDayDWD, Long, GlobalWindow>.Context context,
                        Iterable<AdDataOfDayODS> iterable, Collector<PlanStatOfDayDWD> collector) throws Exception {
        AdDataOfDayODS element = iterable.iterator().next();
        LocalDate statDay = DateUtil.parseLocalDate(element.getStatDay());
        long now = System.currentTimeMillis();

        String lastQueryDay = lastQueryDayState.value();
        // 从 maxCompute查找广告的历史数据
        if (lastQueryDay == null || !lastQueryDay.equals(DateUtil.formatLocalDate(LocalDate.now()))) {
            LocalDate endTime = LocalDate.now(), beginTime = statDay.minusDays(60);
            String sql = "SELECT * FROM plan_stat_of_day_dwd WHERE stat_day >= \"" + DateUtil.formatLocalDate(beginTime) + "\" AND stat_day <= \"" + DateUtil.formatLocalDate(endTime) + "\" AND campaign_id = " + element.getCampaignId() + ";";
            Instance instance = SQLTask.run(odps, sql);
            // log.error("sql: " + sql + ", odps日志: " + odps.logview().generateLogView(instance, 7 * 24));
            instance.waitForSuccess();
            List<Record> records = SQLTask.getResult(instance);
            Map<String, PlanStatOfDayDWD> historyData = records.stream()
                    .map(PlanStatOfDayDWD::byMaxCompute)
                    .sorted((o1, o2) -> new Long(o1.getCreateTime().getTime() - o2.getCreateTime().getTime()).intValue())
                    .collect(Collectors.toMap(PlanStatOfDayDWD::getStatDay, data -> data, (val1, val2) -> val2));
            historyReduceState.clear();
            historyReduceState.putAll(historyData);
            lastQueryDayState.update(DateUtil.formatLocalDate(LocalDate.now()));
        }
        PlanStatOfDayDWD lastReduceData = null;
        for (int i = 1; i <= 60; i++) {
            lastReduceData = historyReduceState.get(DateUtil.formatLocalDate(statDay.minusDays(i)));
            if (lastReduceData != null) {
                break;
            }
        }
        PlanStatOfDayDWD newStatData = PlanStatOfDayDWD.reduce(lastReduceData, element, now);
        historyReduceState.put(DateUtil.formatLocalDate(statDay), newStatData);
        collector.collect(newStatData);
    }
}
